// Definitions for the automated inclusion into the parser
// LABEL:		DeltaPhi_Cap2
// CLASSNAME:	ObservableDeltaPhiCapacitor2
// 
#ifndef OBSERVABLEDELTAPHICAPACITOR2_H
#define OBSERVABLEDELTAPHICAPACITOR2_H

#include "observable.h"
#include "parameter.h"
#include <math.h>


//! Observable for the transmembrane potential of a spherical capacitor.

//! Subclass representing the transmembrane potential deltaPhi,
//! derived from Observable.
//!
//! There is one central parameters:
//! the inverse capacitance in units of [mV / e], i.e., the voltage that is generted by one elementary charge
//!
//! DeltaPhi accesses five pools of charged species: 
//! * for the free protons inside the vesicle, 
//! * for protons bound to the membrane,
//! * for charges  created from internal charge separations in the proteins,
//! * and for difference between numbers in the two cytochrome c pools

class ObservableDeltaPhiCapacitor2 : public Observable
{
	public:
	
	//! Constructor
	ObservableDeltaPhiCapacitor2(const std::string name);
	//! Destructor
	~ObservableDeltaPhiCapacitor2();
	
	//! Calculate and return the value of the transmembrane potential. 
	//DeltaPhi is returned in units of milliVolts [mV]. 
	double get_value();
	
	//! Parser for line from input file
	bool parseLine(char *linep);
	
	
	private:
	
	//! @name Char[]s for the pool names (see length above)
	//@{
	char FreeHPool[MAX_POOLNAME_LENGTH], 
		 MembraneHPool[MAX_POOLNAME_LENGTH], 
		 ProteinHPool[MAX_POOLNAME_LENGTH],
		 c2redPool[MAX_POOLNAME_LENGTH],
		 c2oxPool[MAX_POOLNAME_LENGTH];
	//@}
	 
	//! name pointer to pools and variables for reaction constants to 
	//! avoid their continuous lookup
	//@{
	Pool *FreeHPoolp, *MembraneHPoolp, *ProteinHPoolp, *c2redPoolp, *c2oxPoolp;
	double voltagePerCharge, Q_offset;
	double freeHScaling, membraneHScaling, proteinHScaling, c2redScaling, c2oxScaling;
	//@}
	
	//! verify that all neccessary pools are defined
	void verify_pools();

};
#endif /* OBSERVABLEDELTAPHICAPACITOR_H */
