// Definitions for the automated inclusion into the parser
// LABEL:		OnOff_list
// CLASSNAME:	ObservableOnOffList
// 
#ifndef OBSERVABLE_ONOFFLIST_H
#define OBSERVABLE_ONOFFLIST_H

#include "observable.h"
#include "proteinOnOff.h"
#include "parameter.h"
#include <math.h>
#include <stdlib.h>
#include <stdio.h>


//! Observable that queries the state of the OnOff-protein with index idx.
//! There are no Parameters

class ObservableOnOffList : public Observable
{
	public:
	
	//! Constructor
	ObservableOnOffList(const std::string name);
	//! Destructor
	~ObservableOnOffList() { };

	//! Returning a double for the list of OnOffs is meaningless => return(0)
	double get_value() { return(0.0); }
	
	//! print an X for every protein in "on" state and a "." for "off". 
	std::string get_formattedValue(); 
	
	//! Parser for input line
	bool parseLine(char *linep);
	
	
	private:
		
	//! verify that all neccessary pools are defined -> no pools here
	void verify_pools() { };

	//! max. number of OnOff units to list
	long int maxIdx;
};
#endif /* OBSERVABLE_ONOFFLIST_H */
