// Definitions for the automated inclusion into the parser
// LABEL:		ATPase
// CLASSNAME:	ProteinATPase
// 
#ifndef ProteinATPase_h
#define ProteinATPase_h

#include<string>
#include<map>
#include<stdlib.h>
#include"protein.h"
#include"pool.h"
#include "observable.h"
#include"parameter.h"
#include "random.h"
#include<iostream>
#include<string>
#include<math.h>


//! Class that represents the F0-F1-ATPase

/*!
It is driven by a transmembrane potential between the inside and outside the vesicle. 
The ATPase takes protons from the inside and generates ATP on the outside.
Currently, the ATPase takes 4 H+ per ATP.
The rate is a fit to an experimentally determined characteristic.

Pools: H+inside, H+outside, ADP+P-Pool, ATP-Pool

The ATPase assumes that there is always some ADP available (What does the ATPase do if no ADP is available?).
*/

class ProteinATPase : public Protein
{
	public:
	
	//! Constructor
	ProteinATPase(const std::string name, const double &timestep);
	//! Destructor
	~ProteinATPase();
	
	//! Parser for setup line
	bool parseLine(char *linep);
	
	
	private:
	
	//! @name Reactions of the ATPase
	//@{
	//! Process proton with rate dependent on dPhi, one ATP for every 4 protons
	void reaction1a(Protein *);
	//! Bind a proton from the inside if available
	void reaction1b(Protein *);
	//@}

	//! Defs of the internal registers and binding sites.
	//! (registers = int,  binding sites = bool)
	// @{
	//! register for number of translocated protons
	int reg_H;
	//! binding site for H+ on the inside
	bool bs_Hin;
	// @}

	//! @name Char[]s for the pool and observable names (see length above)
	//@{
	char HinsidePool[MAX_POOLNAME_LENGTH], 
	     HoutsidePool[MAX_POOLNAME_LENGTH], 
		 ADPPool[MAX_POOLNAME_LENGTH], 
		 ATPPool[MAX_POOLNAME_LENGTH];
	char DeltaPhiObs[MAX_POOLNAME_LENGTH];
	//@}
	
	//! Pointers to metabolite pools and observable(s)
	//@{
	Pool *HinsidePoolp, *HoutsidePoolp, *ADPPoolp, *ATPPoolp;
	Observable *DeltaPhiObsp;
	//@}
		
	//! verify that all neccessary pools are defined
	void verify_pools();
	
	//! @name For listing the internals
	// @{
	std::string listInternals();
	void writeInternals();
	// @}
};

#endif /* ProteinATPase_h */

