#include "observable.h"
#include "observableDeltaPhi_Nernst.h"

using namespace std;


//! Constructor
ObservableDeltaPhiNernst::ObservableDeltaPhiNernst(const string &name) :
Observable::Observable(name)
{ }



void ObservableDeltaPhiNernst::verify_pools()
{
	if(!(Pools->count(string(HoutsidePool)))) {
		cerr << "ObservableDeltaPhiNernst::verify_pools: pool "<<HoutsidePool<<" not defined" << endl;
		exit(-3);
	}
	if(!(Pools->count(string(HinsidePool)))) {
		cerr << "ObservableDeltaPhiNernst::verify_pools: pool "<<HinsidePool<<" not defined" << endl;
		exit(-3);
	}
	
	HoutsidePoolp = getPool(HoutsidePool);
	HinsidePoolp  = getPool(HinsidePool);
}



bool ObservableDeltaPhiNernst::parseLine(char *linep)
{
	char  dummy[128];

	// read date into the variables
	if(sscanf(linep, "%s %s %lf %s %s", dummy, dummy, &pHVoltage, HinsidePool, HoutsidePool) != 5) {
		cerr << "Simulation::parseFile: not enough parameters for DeltaPhi line >>" << linep << "<<" << endl;
		exit(-2);
	}
	
	#ifdef SCHWAETZ
	cout << "------------------" << endl;
	cout << "   DeltaPhi Nernst: read: " << pHVoltage << endl;
	cout << "   DeltaPhi Nernst: read: " << HinsidePool << "  " << HoutsidePool << endl;
	#endif
	
	return(true);
}


double ObservableDeltaPhiNernst::get_value()
{
	double deltapH = log(max(1.0, HinsidePoolp->getConcentration()/HoutsidePoolp->getConcentration()));
	double dphi = pHVoltage * deltapH;
	
	#ifdef DEBUG
	cerr << "ObservableDeltaPhiNernst::get_value:  dPhi = " << dphi << endl;
	#endif

	return(dphi);
}
