#include"proteinBC1Dimer.h"

// #define DEBUG
// #define VERBOSE

using namespace std;

ProteinBC1Dimer::ProteinBC1Dimer(const std::string name, const double &timestep) :
Protein::Protein(name, timestep)
{
	// initialize internals
	// dark adapted state: QH2 at Qo, Q at Qi, no c2, H bound and waiting,
	bs_Qo_1 = true;
	bs_Qo_2 = true;
	bs_Qi_1 = true;
	bs_Qi_2 = true;
	bs_c2_1 = false;
	bs_c2_2 = false;
	bs_H_1  = true;
	bs_H_2  = true;
	
	reg_Qo_1  = 2;
	reg_Qo_2  = 2;
	reg_Qie_1 = 0;
	reg_Qie_2 = 0;
	reg_QiH_1 = 0;
	reg_QiH_2 = 0;
	reg_FeS_1 = 1;
	reg_FeS_2 = 1;
	reg_c2_1  = 0;
	reg_c2_2  = 0;
	
   // pointers to functions:
   function fu1;
   
   void (Protein::*pr1) (Protein *) = (void(Protein::*)(Protein *))(&ProteinBC1Dimer::reaction1);
   fu1.pfunction = pr1;
   setFunction("r1", fu1);
   
   void (Protein::*pr2) (Protein *) = (void(Protein::*)(Protein *))(&ProteinBC1Dimer::reaction2);
   fu1.pfunction = pr2;
   setFunction("r2", fu1);
   
   void (Protein::*pr3) (Protein *) = (void(Protein::*)(Protein *))(&ProteinBC1Dimer::reaction3);
   fu1.pfunction = pr3;
   setFunction("r3", fu1);
   
   void (Protein::*pr4) (Protein *) = (void(Protein::*)(Protein *))(&ProteinBC1Dimer::reaction4);
   fu1.pfunction = pr4;
   setFunction("r4", fu1);

   void (Protein::*pr5) (Protein *) = (void(Protein::*)(Protein *))(&ProteinBC1Dimer::reaction5);
   fu1.pfunction = pr5;
   setFunction("r5", fu1);

   void (Protein::*pr6_1) (Protein *) = (void(Protein::*)(Protein *))(&ProteinBC1Dimer::reaction6_1);
   fu1.pfunction = pr6_1;
   setFunction("r6_1", fu1);

   void (Protein::*pr6_2) (Protein *) = (void(Protein::*)(Protein *))(&ProteinBC1Dimer::reaction6_2);
   fu1.pfunction = pr6_2;
   setFunction("r6_2", fu1);

   void (Protein::*pr7_1) (Protein *) = (void(Protein::*)(Protein *))(&ProteinBC1Dimer::reaction7_1);
   fu1.pfunction = pr7_1;
   setFunction("r7_1", fu1);

   void (Protein::*pr7_2) (Protein *) = (void(Protein::*)(Protein *))(&ProteinBC1Dimer::reaction7_2);
   fu1.pfunction = pr7_2;
   setFunction("r7_2", fu1);
   
   void (Protein::*pr8) (Protein *) = (void(Protein::*)(Protein *))(&ProteinBC1Dimer::reaction8);
   fu1.pfunction = pr8;
   setFunction("r8", fu1);
   
   void (Protein::*pr9) (Protein *) = (void(Protein::*)(Protein *))(&ProteinBC1Dimer::reaction9);
   fu1.pfunction = pr9;
   setFunction("r9", fu1);

   void (Protein::*pr10_1) (Protein *) = (void(Protein::*)(Protein *))(&ProteinBC1Dimer::reaction10_1);
   fu1.pfunction = pr10_1;
   setFunction("r10_1", fu1);

   void (Protein::*pr10_2) (Protein *) = (void(Protein::*)(Protein *))(&ProteinBC1Dimer::reaction10_2);
   fu1.pfunction = pr10_2;
   setFunction("r10_2", fu1);
   
   timer_r6_1  = 0;
   timer_r6_2  = 0;
   timer_r7_1  = 0;
   timer_r7_2  = 0;
   timer_r10_1 = 0;
   timer_r10_2 = 0;
}




std::string ProteinBC1Dimer::listInternals() {
	return(string("#time\tbs_Qo_1\tbs_Qi_1\tbs_c2_1\tbs_H_1\treg_Qie_1\treg_QiH_1\treg_Qo_1\treg_FeS_1\treg_c2_1\tbs_Qo_2\tbs_Qi_2\tbs_c2_2\tbs_H_2\treg_Qie_2\treg_QiH_2\treg_Qo_2\treg_FeS_2\treg_c2_2"));
}



void ProteinBC1Dimer::writeInternals()
{
	if(outputInternals)
	{
		double realTime = timer * delta_t;
																   
		internalsOutFile.open(internalFileName.c_str(), fstream::app);
		internalsOutFile << realTime << "\t" << bs_Qo_1 << "\t" << bs_Qi_1 << "\t" << bs_c2_1 << "\t" << bs_H_1 << "\t" << reg_Qie_1 << "\t" << reg_QiH_1 << "\t" << reg_Qo_1 << "\t" << reg_FeS_1 << "\t" << reg_c2_1 
						 << "\t" << bs_Qo_2 << "\t" << bs_Qi_2 << "\t" << bs_c2_2 << "\t" << bs_H_2 << "\t" << reg_Qie_2 << "\t" << reg_QiH_2 << "\t" << reg_Qo_2 << "\t" << reg_FeS_2 << "\t" << reg_c2_2 << "\n";
		internalsOutFile.close();
	}
}



void ProteinBC1Dimer::verify_pools()
{
	if(!Pools->count(string(QoffPool))) {
		cerr << "ProteinBC1Dimer::verify_pools: pool "<<QoffPool<<" not defined" << endl;
		exit(-3);
	}
	if(!Pools->count(string(QH2offPool))) {
		cerr << "ProteinBC1Dimer::verify_pools: pool "<<QH2offPool<<" not defined" << endl;
		exit(-3);
	}
	if(!Pools->count(string(QonPool))) {
		cerr << "ProteinBC1Dimer::verify_pools: pool "<<QonPool<<" not defined" << endl;
		exit(-3);
	}
	if(!Pools->count(string(QH2onPool))) {
		cerr << "ProteinBC1Dimer::verify_pools: pool "<<QH2onPool<<" not defined" << endl;
		exit(-3);
	}
	if(!Pools->count(string(c2redPool))) {
		cerr << "ProteinBC1Dimer::verify_pools: pool "<<c2redPool<<" not defined" << endl;
		exit(-3);
	}
	if(!Pools->count(string(c2oxPool))) {
		cerr << "ProteinBC1Dimer::verify_pools: pool "<<c2oxPool<<" not defined" << endl;
		exit(-3);
	}
	if(!Pools->count(string(HoutsidePool))) {
		cerr << "ProteinBC1Dimer::verify_pools: pool "<<HoutsidePool<<" not defined" << endl;
		exit(-3);
	}
	if(!Pools->count(string(HinsidePool))) {
		cerr << "ProteinBC1Dimer::verify_pools: pool "<<HinsidePool<<" not defined" << endl;
		exit(-3);
	}
	

	if(!Observables->count(string(DeltaPhiObs))) {
		cerr << "ProteinATPase::verify_pools: observable "<<DeltaPhiObs<<" not defined" << endl;
		exit(-3);
	}
	
	QoffPoolp     = getPool(QoffPool);
	QH2offPoolp   = getPool(QH2offPool);
	QonPoolp      = getPool(QonPool);
	QH2onPoolp    = getPool(QH2onPool);
	c2redPoolp    = getPool(c2redPool);
	c2oxPoolp     = getPool(c2oxPool);
	HoutsidePoolp = getPool(HoutsidePool);
	HinsidePoolp  = getPool(HinsidePool);
	
	DeltaPhiObsp  = getObservable(DeltaPhiObs);
}



bool ProteinBC1Dimer::parseLine(char *linep)
{
	char  dummy[128];
	int num_of_proteins;
	
	if(sscanf(linep, "%s %d %s %lf %s %lf %s %lf %s %lf %s %lf %s %lf %s %lf %s %lf %lf %s %lf %lf %lf", 
			  dummy, &num_of_proteins, 
			  QH2onPool, &QH2_kon, QoffPool, &Q_koff, QonPool, &Q_kon, QH2offPool, &QH2_koff, c2oxPool, &c2ox_kon, c2redPool, &c2red_koff, 
			  HoutsidePool, &Houtside_kon, HinsidePool, &k_QoQiSwap, &k_eQoFeS, DeltaPhiObs, &k_eQoFeS_phi0, &k_HQi, &k_FeSSwap) != 23) {
		cerr << "Simulation::parseFile: not enough parameters for bc1dimer line >>" << linep << "<<" << endl;
		return(false);
	}
	// Check signs of the rates
	if((QH2_kon<0) || (Q_koff<0) || (Q_kon<0) || (QH2_koff<0) || (c2ox_kon<0) || (c2red_koff<0) || (Houtside_kon<0) || (k_QoQiSwap<0) || (k_eQoFeS<0) || (k_eQoFeS_phi0<0) || (k_HQi<0) || (k_FeSSwap<0)) {
		cerr << "Simulation::parseFile: negative rate constant in bc1dimer line >>" << linep << "<<" << endl;
		return(false);
	}
	
	#ifdef VERBOSE
	cout << "------------------" << endl;
	cout << "   bc1dimer: read: " << num_of_proteins << endl;
	cout << "   bc1dimer: read: " << QH2onPool << "  " << QH2_kon << endl;
	cout << "   bc1dimer: read: " << QoffPool << "  " << Q_koff << endl;
	cout << "   bc1dimer: read: " << QonPool << "  " << Q_kon << endl;
	cout << "   bc1dimer: read: " << QH2offPool << "  " << QH2_koff << endl;
	cout << "   bc1dimer: read: " << c2oxPool << "  " << c2ox_kon << endl;
	cout << "   bc1dimer: read: " << c2redPool << "  " << c2red_koff << endl;
	cout << "   bc1dimer: read: " << HoutsidePool << "  " << Houtside_kon << endl;
	cout << "   bc1dimer: read: " << HinsidePool << endl;
	cout << "   bc1dimer: read:                " << k_QoQiSwap << endl;
	cout << "   bc1dimer: read: " << DeltaPhiObs << "  " << k_eQoFeS << endl;
	cout << "   bc1dimer: read:                " << k_eQoFeS_phi0 << endl;
	cout << "   bc1dimer: read:                " << k_HQi << endl;
	cout << "   bc1dimer: read:                " << k_FeSSwap << endl;
	#endif

	return(true);
}


// ++++++++++++++ internal reactions ++++++++++++++++++++++++

// binds a QH2 molecule at either of the Qo sites
void ProteinBC1Dimer::reaction1(Protein *p)
{
	if (!bs_Qo_1)
		{
		if (QH2onPoolp->take_out(QH2_kon))
		{
			bs_Qo_1 = true;
			reg_Qo_1 = 2;		// now 2 e-H-pairs can be unloaded
			writeInternals();
			#ifdef DEBUG
			cout << "\tProteinBC1Dimer::reaction1_1: QH2 binding at Qo is done\n";
			#endif
		}
	}  

	if (!bs_Qo_2)
		{
		if (QH2onPoolp->take_out(QH2_kon))
		{
			bs_Qo_2 = true;
			reg_Qo_2 = 2;		// now 2 e-H-pairs can be unloaded
			writeInternals();
			#ifdef DEBUG
			cout << "\tProteinBC1Dimer::reaction1_2: QH2 binding at Qo is done\n";
			#endif
		}
	}  
}

// binds Q molecules at the Qi sites
void ProteinBC1Dimer::reaction2(Protein *p)
{
	if (!bs_Qi_1)
		{
		if (QonPoolp->take_out(Q_kon))
		{
			bs_Qi_1   = true;
			reg_Qie_1 = 0;
			reg_QiH_1 = 0;
			writeInternals();
			#ifdef DEBUG
			cout << "\tProteinBC1Dimer::reaction2_1: Q binding at Qi is done\n";
			#endif
		}
	}

	if (!bs_Qi_2)
		{
		if (QonPoolp->take_out(Q_kon))
		{
			bs_Qi_2   = true;
			reg_Qie_2 = 0;
			reg_QiH_2 = 0;
			writeInternals();
			#ifdef DEBUG
			cout << "\tProteinBC1Dimer::reaction2_2: Q binding at Qi is done\n";
			#endif
		}
	}
}


// binds a oxidized cytochrome c2 at FeS site
// only one c2 can be bound at a time
// => twice the reaction constant
void ProteinBC1Dimer::reaction3(Protein *p)
{
	if ((!bs_c2_1) && (!bs_c2_2))
	{
		if (c2oxPoolp->take_out(2.0 * c2ox_kon))
		{
			// left or right?
			if(lin_ran() < 0.5)
			{
				bs_c2_1  = true;
				reg_c2_1 = 0;
			} else {
				bs_c2_2  = true;
				reg_c2_2 = 0;
			}
			
			writeInternals();
			#ifdef DEBUG
			cout << "\tProteinBC1Dimer::reaction3: c2ox binding is done\n";
			#endif
		}
	}
}


// bind proton from outside the vesicle
void ProteinBC1Dimer::reaction4(Protein *p)
{
	if (!bs_H_1)
	{
		if (HoutsidePoolp->take_out(Houtside_kon))
		{
			bs_H_1 = true;
			writeInternals();
			#ifdef DEBUG
			cout << "\tProteinBC1Dimer::reaction4_1: H+ bound from outside\n";
			#endif
		}
	}
	
	if (!bs_H_2)
	{
		if (HoutsidePoolp->take_out(Houtside_kon))
		{
			bs_H_2 = true;
			writeInternals();
			#ifdef DEBUG
			cout << "\tProteinBC1Dimer::reaction4_2: H+ bound from outside\n";
			#endif
		}
	}
}


// unbinding of emptied Q from Q0 site and possible rebinding at Qi
void ProteinBC1Dimer::reaction5(Protein *p)
{
	if (bs_Qo_1 && (reg_Qo_1 == 0))
	{
		// Q swaps directly to Qi
		if((!bs_Qi_2) && (lin_ran() <= (k_QoQiSwap * delta_t)))
		{
			bs_Qo_1   = false;
			bs_Qi_2   = true;
			reg_Qie_2 = 0;
			reg_QiH_2 = 0;
			writeInternals();
			#ifdef DEBUG
			cout << "\tProteinBC1Dimer::reaction5_1: Qo -> Qi swapped\n";
			#endif
		}
		else if(lin_ran() <= (Q_koff * delta_t))
		{
			QoffPoolp->put_back();
			bs_Qo_1 = false;
			writeInternals();
			#ifdef DEBUG
			cout << "\tProteinBC1Dimer::reaction5_1: Qo back to pool\n";
			#endif
		}
	}
	
	if (bs_Qo_2 && (reg_Qo_2 == 0))
	{
		// Q swaps directly to Qi
		if((!bs_Qi_1) && (lin_ran() <= (k_QoQiSwap * delta_t)))
		{
			bs_Qo_2   = false;
			bs_Qi_1   = true;
			reg_Qie_1 = 0;
			reg_QiH_1 = 0;
			writeInternals();
			#ifdef DEBUG
			cout << "\tProteinBC1Dimer::reaction5_2: Qo -> Qi swapped\n";
			#endif
		}
		else if(lin_ran() <= (Q_koff * delta_t))
		{
			QoffPoolp->put_back();
			bs_Qo_2 = false;
			writeInternals();
			#ifdef DEBUG
			cout << "\tProteinBC1Dimer::reaction5_2: Qo back to pool\n";
			#endif
		}
	}
}

// unbinding of QH2 from Qi site #1
void ProteinBC1Dimer::reaction6_1(Protein *p)
{
	if (bs_Qi_1 && (reg_QiH_1 == 2) && (reg_Qie_1 == 2)) // test for reg_Qie superfluous
	{
		switch(timer_r6_1)
		{
			case 0:
			{
				double t0 = 1.0/(QH2_koff * delta_t);
				timer_r6_1 = static_cast<unsigned long int>(exp_ran(t0));
				#ifdef DEBUG
				cout << "\tProteinBC1Dimer::reaction6_1: QH2 unbinding started: time = " << timer_r6_1 << "\n";
				#endif
			}; break;
			case 1:
			{
				QH2offPoolp->put_back();
				bs_Qi_1   = false;
				reg_Qie_1 = 0;
				reg_QiH_1 = 0;
				writeInternals();
				timer_r6_1 = 0;
				#ifdef DEBUG
				cout << "\tProteinBC1Dimer::reaction6_1: QH2 unbinding is done\n";
				#endif
			}; break;
			default: 
				timer_r6_1--;
		}
	}
}

// unbinding of QH2 from Qi site #2
void ProteinBC1Dimer::reaction6_2(Protein *p)
{
	if (bs_Qi_2 && (reg_QiH_2 == 2))
	{
		switch(timer_r6_2)
		{
			case 0:
			{
				double t0 = 1.0/(QH2_koff * delta_t);
				timer_r6_2 = static_cast<unsigned long int>(exp_ran(t0));
				#ifdef DEBUG
				cout << "\tProteinBC1Dimer::reaction6_2: QH2 unbinding started: time = " << timer_r6_2 << "\n";
				#endif
			}; break;
			case 1:
			{
				QH2offPoolp->put_back();
				bs_Qi_2 = false;
				reg_Qie_2 = 0;
				reg_QiH_2 = 0;
				writeInternals();
				timer_r6_2 = 0;
				#ifdef DEBUG
				cout << "\tProteinBC1Dimer::reaction6_2: QH2 unbinding is done\n";
				#endif
			}; break;
			default: 
				timer_r6_2--;
		}
	}
}


// unbinding of c2 from c2 site #1
void ProteinBC1Dimer::reaction7_1(Protein *p)
{
	if (bs_c2_1 && (reg_c2_1 == 1))
	{
		switch(timer_r7_1)
		{
			case 0:
			{
				double t0 = 1.0/(c2red_koff * delta_t);
				timer_r7_1 = static_cast<unsigned long int>(exp_ran(t0));
				#ifdef DEBUG
				cout << "\tProteinBC1Dimer::reaction7_1: c2 unbinding started: time = " << timer_r7_1 << "\n";
				#endif
			}; break;
			case 1:
			{
				c2redPoolp->put_back();
				bs_c2_1  = false;
				reg_c2_1 = 0;
				writeInternals();
				timer_r7_1 = 0;
				#ifdef DEBUG
				cout << "\tProteinBC1Dimer::reaction7_1: c2 unbinding is done\n";
				#endif
			}; break;
			default: 
				timer_r7_1--;
		}
	}
}

// unbinding of c2 from c2 site #2
void ProteinBC1Dimer::reaction7_2(Protein *p)
{
	if (bs_c2_2 && (reg_c2_2 == 1))
	{
		switch(timer_r7_2)
		{
			case 0:
			{
				double t0 = 1.0/(c2red_koff * delta_t);
				timer_r7_2 = static_cast<unsigned long int>(exp_ran(t0));
				#ifdef DEBUG
				cout << "\tProteinBC1Dimer::reaction7_2: c2 unbinding started: time = " << timer_r7_2 << "\n";
				#endif
			}; break;
			case 1:
			{
				c2redPoolp->put_back();
				bs_c2_2 = false;
				reg_c2_2 = 0;
				writeInternals();
				timer_r7_2 = 0;
				#ifdef DEBUG
				cout << "\tProteinBC1Dimer::reaction7_2: c2 unbinding is done\n";
				#endif
			}; break;
			default: 
				timer_r7_2--;
		}
	}
}
		


// electron transfer from Qo -> FeS + Qi
// this reaction is limited by \deltaPhi
void ProteinBC1Dimer::reaction8(Protein *p)
{
	if((reg_FeS_1 == 0) && bs_Qo_1 && (reg_Qo_1 == 2) && bs_Qi_1 && (reg_Qie_1 < 2))
	{
		double dPhi = DeltaPhiObsp->get_value();
		double rate = k_eQoFeS * delta_t * exp(-dPhi / k_eQoFeS_phi0);
		
		if(lin_ran() < rate)
		{
			reg_Qo_1 = 0;
			reg_FeS_1++;
			reg_Qie_1++;
			HinsidePoolp->put_back();
			HinsidePoolp->put_back();
			writeInternals();
			#ifdef DEBUG
			cout << "\tProteinBC1Dimer::reaction8_1: electron transfer Qo -> FeS/Qi done\n";
			#endif
		}
	}
	
	if((reg_FeS_2 == 0) && bs_Qo_2 && (reg_Qo_2 == 2) && bs_Qi_2 && (reg_Qie_2 < 2))
	{
		double dPhi = DeltaPhiObsp->get_value();
		double rate = k_eQoFeS * delta_t * exp(-dPhi / k_eQoFeS_phi0);

		if(lin_ran() < rate)
		{
			reg_Qo_2 = 0;
			reg_FeS_2++;
			reg_Qie_2++;
			HinsidePoolp->put_back();
			HinsidePoolp->put_back();
			writeInternals();
			#ifdef DEBUG
			cout << "\tProteinBC1Dimer::reaction8_2: electron transfer Qo -> FeS/Qi done\n";
			#endif
		}
	}
}

// fill up the proton on the Qi
void ProteinBC1Dimer::reaction9(Protein *p)
{
	if(bs_H_1 && (reg_QiH_1 < reg_Qie_1))
	{
		if(lin_ran() < (k_HQi * delta_t))
		{
			bs_H_1 = false;
			reg_QiH_1++;
			writeInternals();
			#ifdef DEBUG
			cout << "\tProteinBC1Dimer::reaction9_1: proton transfer to Qi done\n";
			#endif
		}
	}
	
	if(bs_H_2 && (reg_QiH_2 < reg_Qie_2))
	{
		if(lin_ran() < (k_HQi * delta_t))
		{
			bs_H_2 = false;
			reg_QiH_2++;
			writeInternals();
			#ifdef DEBUG
			cout << "\tProteinBC1Dimer::reaction9_2: proton transfer to Qi done\n";
			#endif
		}
	}
}

// Movement of the FeS domain and electron transfer onto the bound c2
void ProteinBC1Dimer::reaction10_1(Protein *p)
{
	if (bs_c2_1 && (reg_FeS_1==1) && (reg_c2_1==0))
	{
		switch(timer_r10_1)
		{
			case 0:
			{
				double t0 = 1.0/(k_FeSSwap * delta_t);
				timer_r10_1 = static_cast<unsigned long int>(exp_ran(t0));
				#ifdef DEBUG
				cout << "\tProteinBC1Dimer::reaction10_1: FeS swap started: time = " << timer_r10_1 << "\n";
				#endif
			}; break;
			case 1:
			{
				reg_FeS_1--;
				reg_c2_1++;
				writeInternals();
				timer_r10_1 = 0;
				#ifdef DEBUG
				cout << "\tProteinBC1Dimer::reaction10_1: FeS swap is done\n";
				#endif
			}; break;
			default: 
				timer_r10_1--;
		}
	}
}

void ProteinBC1Dimer::reaction10_2(Protein *p)
{
	if (bs_c2_2 && (reg_FeS_2==1) && (reg_c2_2==0))
	{
		switch(timer_r10_2)
		{
			case 0:
			{
				double t0 = 1.0/(k_FeSSwap * delta_t);
				timer_r10_2 = static_cast<unsigned long int>(exp_ran(t0));
				#ifdef DEBUG
				cout << "\tProteinBC1Dimer::reaction10_2: FeS swap started: time = " << timer_r10_2 << "\n";
				#endif
			}; break;
			case 1:
			{
				reg_FeS_2--;
				reg_c2_2++;
				writeInternals();
				timer_r10_2 = 0;
				#ifdef DEBUG
				cout << "\tProteinBC1Dimer::reaction10_2: FeS swap is done\n";
				#endif
			}; break;
			default: 
				timer_r10_2--;
		}
	}
}

