#include "proteinConnector.h"

// #define DEBUG
// #define VERBOSE

using namespace std;

ProteinConnector::ProteinConnector(const std::string name, const double &timestep) :
Protein::Protein(name, timestep)
{
	function fu1;
	
	void (Protein::*pr1)(Protein *) = (void(Protein::*)(Protein *))(&ProteinConnector::reaction1);
	fu1.pfunction = pr1;
	setFunction("r1", fu1);
}


// @todo Directly use pool name chars for parsing, get rid of intermediate poolX vars
bool ProteinConnector::parseLine(char *linep)
{
	char  dummy[128]; // , pool0[128], pool1[128];
	int idx;

	// read data into the variables
	if(sscanf(linep, "%s %d %s %lf %s %lf", dummy, &idx, Pool1, &Pool1_kon, Pool2, &Pool2_kon) != 6) {
		cerr << "ProteinConnector::parseLine: not enough parameters for Connector line >>" << linep << "<<" << endl;
		return(false);
	}
	// Check signs of the rates
	if((Pool1_kon < 0.0) || (Pool2_kon < 0.0)) {
		cerr << "ProteinConnector::parseLine: negative rate constant in Connector line >>" << linep << "<<" << endl;
		return(false);
	}
	
	#ifdef VERBOSE			
	cout << "------------------" << endl;
	cout << "   Connector: read: " << idx << endl;
	cout << "   Connector: read: " << Pool1 << "  " << Pool1_kon << endl;
	cout << "   Connector: read: " << Pool2 << "  " << Pool2_kon << endl;
	#endif

	return(true);
}


void ProteinConnector::verify_pools()
{
	if(!(Pools->count(string(Pool1)))) {
		cerr << "ProteinConnector::verify_pools: pool "<<Pool1<<" not defined" << endl;
		exit(-3);
	}
	Pool1p = getPool(Pool1);

	if(!(Pools->count(string(Pool2)))) {
		cerr << "ProteinConnector::verify_pools: pool "<<Pool2<<" not defined" << endl;
		exit(-3);
	}
	Pool2p = getPool(Pool2);
}



// ++++++++++++++ internal reaction ++++++++++++++++++++++++

void ProteinConnector::reaction1(Protein *p)
{
	bool out1 = Pool1p->take_out(Pool1_kon);
	bool out2 = Pool2p->take_out(Pool2_kon);
	
	if(out1) {
		#ifdef VERBOSE
		cerr << "ProteinConnector::r1: transfer one particle to pool2 (" << Pool2 << ")" << endl;
		#endif
		Pool2p->put_back();
	}
	if(out2) {
		#ifdef VERBOSE
		cerr << "ProteinConnector::r1: transfer one particle to pool1 (" << Pool1 << ")" << endl;
		#endif
		Pool1p->put_back();
	}
}
