#include "proteinLHC.h"

// #define DEBUG

using namespace std;

ProteinLHC::ProteinLHC(const std::string name, const double &timestep) :
Protein::Protein(name, timestep)
{
	// define functions
	function fu1;
	
	void (Protein::*pr1)(Protein *) = (void(Protein::*)(Protein *))(&ProteinLHC::reaction1);
	fu1.pfunction = pr1;
	setFunction("r1", fu1);
	
	void (Protein::*pr2)(Protein *) = (void(Protein::*)(Protein *))(&ProteinLHC::reaction2);
	fu1.pfunction = pr2;
	setFunction("r2", fu1);
}


bool ProteinLHC::parseLine(char *linep)
{
	char  dummy[128];
	int num_of_proteins;

	// read date into the variables
	if(sscanf(linep, "%s %d %s %lf %s %lf %lf", dummy, &num_of_proteins, LightPool, &Light_kon, ExcitonPool, &Exciton_kon, &N0_Exciton) != 7) {
		cerr << "ProteinLHC::parseLine: not enough parameters for LHC line >>" << linep << "<<" << endl;
		return(false);
	}
	
	// Check signs of the rates
	if((Light_kon < 0.0) || (Exciton_kon < 0.0) || (N0_Exciton < 0.0)) {
		cerr << "ProteinLHC::parseLine: negative rate constant in LHC line >>" << linep << "<<" << endl;
		return(false);
	}
	
	#ifdef SCHWAETZ			
	cout << "------------------" << endl;
	cout << "   LHC: read: " << num_of_proteins << endl;
	cout << "   LHC: read: " << LightPool << "  " << Light_kon << endl;
	cout << "   LHC: read: " << ExcitonPool << "  " << Exciton_kon << endl;
	cout << "   LHC: read:        " << N0_Exciton << endl;
	#endif

	return(true);
}


void ProteinLHC::verify_pools()
{
	if(!(Pools->count(string(LightPool)))) {
		cerr << "ProteinLHC::verify_pools: pool "<<LightPool<<" not defined" << endl;
		exit(-3);
	}
	if(!(Pools->count(string(ExcitonPool)))) {
		cerr << "ProteinLHC::verify_pools: pool "<<ExcitonPool<<" not defined" << endl;
		exit(-3);
	}
	
	LightPoolp   = getPool(LightPool);
	ExcitonPoolp = getPool(ExcitonPool);
}



// ++++++++++++++ internal reactions ++++++++++++++++++++++++

// Take an exciton from the pool to simulate their decay
void ProteinLHC::reaction1(Protein *p)
{
	if (ExcitonPoolp->take_out(Exciton_kon))
	{
		; // do nuttin', exciton is gone
		#ifdef DEBUG
		cout << "*** ProteinLHC::r2 captured exciton "<< Exciton_kon << endl;
		#endif
	}
}


// Capture photons (with bleaching)
void ProteinLHC::reaction2(Protein *p)
{
	// get exciton number in the pool and determine reduced rate
	double reduce_exponent = 0.0;
	double already_there = static_cast<double>(ExcitonPoolp->getInside());
	if(already_there > 0.0) {
		reduce_exponent = already_there / N0_Exciton;
	}
	double reduced_rate = Light_kon * exp(-reduce_exponent);
	
	#ifdef DEBUG
	cout << "*** ProteinLHC::r1: already_there = " << already_there << "   rate = " << reduced_rate << endl;
	#endif
	
	// Capture a photon and release it to the exciton pool
	if (LightPoolp->take_out(reduced_rate))
	{
		// photon is put_back as exciton
		ExcitonPoolp->put_back(); 
		#ifdef DEBUG
		cout << "*** ProteinLHC::r1 captured photon "<< reduced_rate << endl;
		#endif
	}
}

