#include "proteinOnOff.h"

// #define DEBUG

using namespace std;

ProteinOnOff::ProteinOnOff(const std::string name, const double &timestep) :
Protein::Protein(name, timestep)
{
	function fu1;
	
	void (Protein::*pr1)(Protein *) = (void(Protein::*)(Protein *))(&ProteinOnOff::reaction1);
	fu1.pfunction = pr1;
	setFunction("R1", fu1);
}


bool ProteinOnOff::parseLine(char *linep)
{
	char  dummy[128]; // , obs0[128];
	int num_of_proteins;

	// read date into the variables
	if(sscanf(linep, "%s %d %lf %lf %s %ld %lf %ld %lf", dummy, &num_of_proteins, &kon, &koff, neighborStateObs, &idxL, &kL, &idxR, &kR) != 9) {
		cerr << "ProteinOnOff::parseLine: not enough parameters for OnOff line >>" << linep << "<<" << endl;
		return(false);
	}
	// check signs of the indices
	if((idxL < 0) || (idxR < 0)) {
		cerr << "ProteinOnOff::parseLine: negative indices in OnOff line >>" << linep << "<<" << endl;
		return(false);
	}
	// check for positive rates
	if((kon < 0.0) || (koff < 0.0) || (kR < 0.0) || (kL < 0.0)) {
		cerr << "ProteinOnOff::parseLine: negative rate constants in OnOff line >>" << linep << "<<" << endl;
		return(false);
	}
	
	#ifdef SCHWAETZ			
	cout << "------------------" << endl;
	cout << "   OnOff: read: " << num_of_proteins << endl;
	cout << "   OnOff: read: " << kon << "  " << koff << endl;
	cout << "   OnOff: read: " << neighborStateObs << endl;
	cout << "   OnOff: read: " << idxL << "  " << kL << endl;
	cout << "   OnOff: read: " << idxR << "  " << kR << endl;
	#endif

	return(true);
}


void ProteinOnOff::verify_pools()
{
	if(!(Observables->count(string(neighborStateObs)))) {
		cerr << "ProteinOnOff::verify_pools: observable "<<neighborStateObs<<" not defined" << endl;
		exit(-3);
	}
	
	neighborStateObsp = getObservable(neighborStateObs);
}



// ++++++++++++++ internal reactions ++++++++++++++++++++++++

void ProteinOnOff::reaction1(Protein *p)
{
	// may switch on when myState if off
	if(!myState) {
		// determine probability, ask neighbors
		double P = delta_t*(kon + kL*neighborStateObsp->get_value(idxL) + kR*neighborStateObsp->get_value(idxR));
		// does something happen?
		if(lin_ran() <= P) {
			myState = true;
		}
	}
	// flip back when on
	else {
		switch(timer_r1)
		{
			case 0:
			{
				// start timer
				double t0 = 1.0 / (koff * delta_t);
				timer_r1 = static_cast<unsigned long int>(exp_ran(t0));
			}; break;
			// perform reaction
			case 1:
			{
				myState = false;
				timer_r1 = 0;
			}; break;
			// count down
			default:
				timer_r1--;
		}
	}
}
