#include"protein.h"

// #define VERBOSE
// #define DEBUG

using namespace std;


// constructor
Protein::Protein(const std::string name, const double &timestep) :
Pools() //, ReactionConstants()
{
	myName = name;
	setTimer(0);
	outputInternals = false;
	internalFileName = "";

	delta_t = timestep;
	
	#ifdef DEBUG
	cout << "Protein::Protein: myName = " << myName << endl;
	#endif
}


// // copy constructor
// Protein::Protein(const Protein& aProtein) :
// Pools(aProtein.Pools),
// ReactionConstants(aProtein.ReactionConstants)
// // Internals(aProtein.Internals),
// // BindingSites(aProtein.BindingSites)
// {
// 	myName = aProtein.myName;
// 	setTimer(0);
// 	outputInternals = aProtein.outputInternals;
// 	internalFileName = aProtein.internalFileName;
// }


// destructor
Protein::~Protein() { }


void Protein::timestep()
{
	map<std::string, function>::iterator iter;
	for (iter = Functions.begin(); iter != Functions.end() ; iter++) {
		(this->*(iter->second).pfunction)(this);
	}
	
	timer++;
}



void Protein::setTimer(unsigned long int value) {
	timer = value;
}


void Protein::setFunction(std::string name, function value) {
	Functions[name] = value;
}


void Protein::setPoolsObservables(std::map<std::string, Pool*>* ptrPools_, std::map<std::string, Observable*>* ptrObservables_) {
	Pools = ptrPools_;
	Observables = ptrObservables_;
	verify_pools();
}


Pool* Protein::getPool(std::string name) {
	return (*Pools)[name];
}

Observable* Protein::getObservable(std::string name) {
	return (*Observables)[name];
}


void Protein::initInternalOutput(const std::string fileName) {
	if(this->listInternals() != "") 
	{
		#ifdef VERBOSE
		cout << "Protein::initInternalOutput: set filename for internals output to " << fileName << endl;
		#endif
	
		internalFileName = fileName;
		
		outputInternals = true;
		
		internalsOutFile.open(internalFileName.c_str(), fstream::out);
		internalsOutFile << this->listInternals() << endl;
		internalsOutFile.close();
		this->writeInternals();
	}
}

std::string Protein::listInternals() {
	#ifdef DEBUG
	cout << "Protein::listInternals(): no description string" << endl;
	#endif
	return(string(""));
}


void Protein::writeInternals() 
{
	if(outputInternals) 
	{
		double realTime = timer * delta_t;
		#ifdef VERBOSE
		cout << "Protein::writeInternals: timer = " << timer << "  real time = "<< realTime << endl;
		#endif
		
		internalsOutFile.open(internalFileName.c_str(), fstream::app);
		internalsOutFile << realTime << endl;
		internalsOutFile.close();
	}
}
